unit UBoolParser12;

interface

uses
  Windows, Classes, SysUtils, afParser12;

type
  TBoolParser12 = class(TParser12)
  private
    function CalcBool: BOOL;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Bool: BOOL read CalcBool stored false;
  end;

procedure Register;

function GetIndexOfType(items: TStrings; vType: Integer): integer;
function ExtRemoveBlanks(const s: string): string;
function StringToVarType(sType: string): integer;
function VarTypeToString(vType: integer): string;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TBoolParser12]);
end;

function VarTypeToString(vType: integer): string;
begin
  case vType and varTypeMask of
    varEmpty: Result := 'Empty';
    varNull: Result := 'Null';
    varSmallint: Result := 'SmallInt';
    varInteger: Result := 'Integer';
    varSingle: Result := 'Single';
    varDouble: Result := 'Double';
    varCurrency: Result := 'Currency';
    varDate: Result := 'Date';
    varOleStr: Result := 'OleStr';
    varDispatch: Result := 'Dispatch';
    varError: Result := 'Error';
    varBoolean: Result := 'Boolean';
    varVariant: Result := 'Variant';
    varUnknown: Result := 'Unknown';
    varByte: Result := 'Byte';
    varString: Result := 'String';
  end;
end;        

function StringToVarType(sType: string): integer;
begin
  if CompareText(sType, 'String') = 0 then Result := varString
  else
    if CompareText(sType, 'Boolean') = 0 then Result := varBoolean
    else
      if CompareText(sType, 'Variant') = 0 then Result := varVariant
      else
        if CompareText(sType, 'Single') = 0 then Result := varSingle
        else
          if CompareText(sType, 'Double') = 0 then Result := varDouble
          else
            if CompareText(sType, 'Date') = 0 then Result := varDate
            else
              if CompareText(sType, 'Empty') = 0 then Result := varEmpty
              else
                if CompareText(sType, 'Null') = 0 then Result := varNull
                else
                  if CompareText(sType, 'SmallInt') = 0 then Result := varSmallint
                  else
                    if CompareText(sType, 'Integer') = 0 then Result := varInteger
                    else
                      if CompareText(sType, 'Currency') = 0 then Result := varCurrency
                      else
                        if CompareText(sType, 'OleStr') = 0 then Result := varOleStr
                        else
                          if CompareText(sType, 'Dispatch') = 0 then Result := varDispatch
                          else
                            if CompareText(sType, 'Error') = 0 then Result := varError
                            else
                              if CompareText(sType, 'Unknown') = 0 then Result := varUnknown
                              else
                                if CompareText(sType, 'Byte') = 0 then Result := varByte;
end;

function GetIndexOfType(items: TStrings; vType: Integer): integer;
begin
  Result := items.IndexOf(VarTypeToString(vType));
end;

function ExtRemoveBlanks(const s: string): string;
var
  counter1, counter2: integer;
  fRange: boolean;
begin
      { remove leading and trailing spaces }
  counter1 := 1;
  counter2 := length(s);
  while counter1 <= counter2 do
    if s[counter1] <> ' ' then
      break
    else
      inc(counter1);

  counter2 := length(s);
  while counter2 > counter1 do
    if s[counter2] <> ' ' then
      break
    else
      dec(counter2);

  Result := Copy(s, counter1, counter2 - counter1 + 1);
  counter1 := 1;
  counter2 := length(Result);

  {remove unneeded spaces and make uppercase}
  fRange := False;
  while counter1 <= counter2 do begin
    if Result[counter1] = #39 then begin
      FRange := not FRange;
    end else
      if not fRange then begin
        if Result[counter1] in [#10, #13, #32, #8, #9, #160] then begin
          delete(Result, counter1, 1);
          dec(Counter2);
          continue;
        end else
          if Result[counter1] in ['a'..'z'] then
            Result[counter1] := char(ord(Result[counter1]) - 32);
      end;
    inc(counter1);
  end;
end;

{ TBoolParser }

function TBoolParser12.CalcBool: BOOL;
begin
  Result := (Value <> 0);
end;

constructor TBoolParser12.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FunctionTwo.AddObject('GREATER', TObject(@_greater));
  FunctionTwo.AddObject('SMALLER', TObject(@_Lesser));
  FunctionTwo.AddObject('OR', TObject(@_lor));
  FunctionTwo.AddObject('AND', TObject(@_land));
  FunctionTwo.AddObject('XOR', TObject(@_lxor));
  FunctionTwo.AddObject('EQUAL', TObject(@_equal));
  FunctionTwo.AddObject('NOTEQUAL', TObject(@_notequal));
  FunctionOne.AddObject('NOT', TObject(@_lnot))
end;

end.

